#======================  Personal Space Project =============================================
#======================  Experiment 2 Mmovies singles with non-choice manipulation  =============================================

# This file is for:
#   => movies singles
#   => sub-sample with tasks WITH no-choice first and WITHOUT no-choice option second as blocks
#   => total of 24 tasks per respondent (12 in each block)
#   => no covariates
#   => based on internal file: PS_Study4_Input transformation NEW with regions UPDATED 2018-01-19.R

# generates files:
#     E2-Movie-Singles.NC/FC:NC
#     E2-Movie-Singles.NC/FC:FC

# Files E2-Movie-Singles.FC/NC:FC and E2-Movie-Singles.FC/NC:NC can be geenrated in a similar 
# way based on the respective CSV files on GitHub


###  ----------------------------------------------------------------------------------------
### Generate input for the hierarchical model estimation (lgtdata)
# ----------------------------------------------------------------------------------------

# Continuous X for rows and columns

dataSeatsNCF <- read.csv("nochoicefirst.csv")
dataLayoutNCF <- read.csv("nochoicefirst_layout.csv")

length(levels(factor(dataLayoutNCF$ParticipantID) ) )  #319
length(levels(factor(dataSeatsNCF$ParticipantID) ) )   #319
list_PIDs_seatsNCF = levels(factor(dataLayoutNCF$ParticipantID) )  #319

#-------------------------------------------------------------------------------------------------------
# generate list with locational X in original scale/coding
#-------------------------------------------------------------------------------------------------------

# check for possible error with misssing data
error_list = NULL; missingY=NULL; count_err_rec=0

#list with locational X in original scale/coding
lgtdata_cont_NCF = list()  
j=1
for(h in 1:length(list_PIDs_seatsNCF))    # for each respondent
{     dataset_h = dataSeatsNCF[which(dataSeatsNCF$ParticipantID==list_PIDs_seatsNCF[h]),]
      dataset_h_available = dataset_h[which(dataset_h[,7]==1),]
      layouts = as.numeric(levels(factor(dataset_h_available[,3]) ))
      layout_skipflag = cbind(layouts,rep(0,length(layouts)))
      for(i in 1:length(layouts)) { layout_skipflag[i,2] = min(dataLayoutNCF[which(dataLayoutNCF[,3]==layouts[i]),12]) }
      lgtdata = list()
      m=1
      for(record in 1:length(layouts))  # for each record within respondents
      { y=NULL
        choiceset = dataset_h_available[which(dataset_h_available[,3]==as.numeric(layouts[record])),]
        if(nrow(choiceset)>0)
        { X=choiceset[,c(6,5)]               # select row and columns numbers
          rownames(X) = c(1:nrow(choiceset)) # choiceset size
          y = which(choiceset[,8]==1)
          if(length(y)==1)
          { if(layout_skipflag[record,2]==1) { X=rbind(X,c(0,0)) }
            lgtdata[[m]]=list( y=y, X=X, p=ifelse(layout_skipflag[record,2]==1,nrow(choiceset)+1,nrow(choiceset)),
                               choicenumber=record, PID=list_PIDs_seatsNCF[h], LayoutID = layouts[record])
             m=m+1
          }
          else{  if(length(y)>1) { count_err_rec=count_err_rec+1
                                   error_list = rbind(error_list,c(list_PIDs_seatsNCF[h],dataset_h[record,1],record,layouts[record]))
                 }
                 else{ 
                   if(layout_skipflag[record,2]==1)
                   {  lgtdata[[m]]=list( y=nrow(choiceset)+1, X=rbind(X,c(0,0)), p=nrow(choiceset)+1, 
                                        choicenumber=record, PID=list_PIDs_seatsNCF[h], LayoutID = layouts[record])
                       m=m+1
                   }
                   else{   missingY = rbind(missingY,c(dataSeatsNCF$Participant_PID ))  }
                 }
          }
        }
        else {  count_err_rec=count_err_rec+1
                error_list = rbind(error_list,c(list_PIDs_seatsNCF[h],dataset_h[record,1],record,dataset_h[record,3]))
        }
      }
      lgtdata_cont_NCF[[j]] = lgtdata
      j=j+1 
}
# takes up to 1 min to generate
count_err_rec; error_list; missingY
length(lgtdata_cont_NCF)   # 319 respondents

# check the distribution of respondents with different number of choices
nchoice_NCF = cbind(c(1:319),rep(0,319))
for(i in 1:length(lgtdata_cont_NCF)) {nchoice_NCF[i,] = length(lgtdata_cont_NCF[[i]])}
table(nchoice_NCF[,2])
# 300 respondents answeres all 24 choice

#-------------------------------------------------------------------------------------------------------
# generate list of PS components
#-------------------------------------------------------------------------------------------------------

lgtdata_XL_original_withPS_regionsUpdated_NCF = list()
Nrows = 12;  Ncols = 12
#test = function(a){
for(h in 1:length(lgtdata_cont_NCF))    # for each respondent
{
     dataResp_h = dataSeatsNCF[which(dataSeatsNCF$ParticipantID==list_PIDs_seatsNCF[h]),]   # all data for one respondent
     dataResp_h = dataResp_h[c(which(dataset_h[,7]==1)),]                                   # remove seats that are not available
     lgtdata_XL_original_withPS_regionsUpdated_NCF[[h]] =list()
     nobs = length(lgtdata_cont_NCF[[h]])
     for(choicetask in 1:nobs)                          # for each choicetask
      {
        X_hh_task = lgtdata_cont_NCF[[h]][[choicetask]]$X           # select all seats (pair coding) for the choice task
        # select all seats available in the room for the choice task
        X_hh_task_allavailable = dataResp_h[which(dataResp_h$LayoutID==lgtdata_cont_NCF[[h]][[choicetask]]$LayoutID),] 
        choiceset=lgtdata_cont_NCF[[h]][[choicetask]]$p
        # create seats table
        seats_occupied = matrix(1, ncol = Ncols+6, nrow = Nrows+6)
        seats_occupied[,1:3] = seats_occupied[,(Ncols+4):(Ncols+6)] = 0
        seats_occupied[1:3,] = seats_occupied[(Nrows+4):(Nrows+6),] = 0
        for(i in 4:(Nrows+3))
        { seatsavailable = sort(X_hh_task_allavailable[X_hh_task_allavailable[,6]==(i-3),5]) +3
          seats_occupied[i,seatsavailable] = 0
        }
        #X_hh_PS=matrix(0,ncol=8,nrow=nrow(X_hh_task))  # used intially but switched to more inclusive PS2
        X_hh_PS2=matrix(0,ncol=15,nrow=nrow(X_hh_task))
       # 1, 2,  3,      4,       5,        6,             7,          8,                9,               10,            11,      12,     13,      14,        15
        # L, R, intLR, countF, countB,  dummyAisleLR, dummyFirstRow, dummyLastRow
        # L, R, intLR, countF, countB,  dummyAisleLR, dummyFirstRow, dummyLastRow, countFrontCorners, countBackCorners, propR1, propR2, propR3, incrR2prop, incrR3prop
        for(choice in 1:(nrow(X_hh_task)))
        { rnumber = X_hh_task[choice,1]         # unshifted
          if(rnumber>0) {  # in case it is the non-choice row for some choices
            cnumber = X_hh_task[choice,2]       # unshifted
            X_hh_PS2[choice,1] = seats_occupied[rnumber+3,cnumber+2]               # L:  
            X_hh_PS2[choice,2] = seats_occupied[rnumber+3,cnumber+4]               # R:  
            X_hh_PS2[choice,4] = seats_occupied[rnumber+2,cnumber+3]               # countF:  
            X_hh_PS2[choice,5] = seats_occupied[rnumber+4,cnumber+3]               # countB:  
            X_hh_PS2[choice,6] = ifelse(cnumber==1,0,ifelse(cnumber==Ncols,0,1))   # dummyAisleLR: 
                    #(!!! coding is different for Aisles to make the reference point, 0=no one is arround)
            X_hh_PS2[choice,7] = ifelse(rnumber==1,0,1)                            # dummyFirstRow:  
                    #(!!! coding is different for Aisles to make the reference point, 0=no one is arround) 
            X_hh_PS2[choice,8] = ifelse(rnumber==Nrows,1,0)                        # dummyLastRow:  
            X_hh_PS2[choice,9] = seats_occupied[rnumber+2,cnumber+2] + seats_occupied[rnumber+2,cnumber+4]   # countFrontCorners:  
            X_hh_PS2[choice,10] = seats_occupied[rnumber+4,cnumber+2] + seats_occupied[rnumber+4,cnumber+4]  # countBackCorners:  
            X_hh_PS2[choice,11] = sum(seats_occupied[(rnumber+2):(rnumber+4),(cnumber+2):(cnumber+4)])/8    ## Region 1 proportion busy
            X_hh_PS2[choice,12] = sum(seats_occupied[(rnumber+1):(rnumber+5),(cnumber+1):(cnumber+5)])/24   ## Region 2 proportion busy
            X_hh_PS2[choice,13] = sum(seats_occupied[(rnumber):(rnumber+6),(cnumber):(cnumber+6)])/48       ## Region 3 proportion busy
            X_hh_PS2[choice,14] = (sum(seats_occupied[(rnumber+1):(rnumber+5),(cnumber+1):(cnumber+5)])-sum(seats_occupied[(rnumber+2):(rnumber+4),(cnumber+2):(cnumber+4)])) /16    ## Region 2 incrememntal proportion
            X_hh_PS2[choice,15] = (sum(seats_occupied[(rnumber):(rnumber+6),(cnumber):(cnumber+6)]) - sum(seats_occupied[(rnumber+1):(rnumber+5),(cnumber+1):(cnumber+5)]))/24       ## Region 3 incrememntal proportion
          }
        }
        X_hh_PS2[,3] = X_hh_PS2[,1]*X_hh_PS2[,2]                  # intLR
        #X_hh_PS2 = cbind(c(rep(0,(nrow(X_hh_task)-1)),1),X_hh_PS2)     # ADD NON-CHOICE intercept if neededm but done later for this file
        lgtdata_XL_original_withPS_regionsUpdated_NCF[[h]][[choicetask]]=list(X_PS2=X_hh_PS2, 
                          p=nrow(X_hh_PS2),  choicenumber=lgtdata_cont_NCF[[h]][[choicetask]]$choicenumber,
                          PID=lgtdata_cont_NCF[[h]][[choicetask]]$PID,LayoutID = lgtdata_cont_NCF[[h]][[choicetask]]$LayoutID) 
      } 
}
#}
# takes about 2-3 min to get it  
# X_PS is now redundant and will be ignore moving forward

#-------------------------------------------------------------------------------------------------------
# take locational and PS (COMBINE) while recode locational X to 0 to 1 scale
#-------------------------------------------------------------------------------------------------------
 
lgtdata_LPS_XL01_NCF = lgtdata_cont_NCF
Nrows = 12;  Ncols = 12
for(h in 1:length(lgtdata_LPS_XL01_NCF))    # for each respondent
{
  for(choicetask in 1:length(lgtdata_LPS_XL01_NCF[[h]]))
  {  Xc = lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$X                    #  (row, col)
     Xc1 = as.numeric(1/(Nrows*2) + (Xc[,1]-1)/Nrows)
     Xc2 = as.numeric(1/(Ncols*2) + (Xc[,2]-1)/Ncols)
     lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$X_L = cbind(Xc1,Xc2)
     p=lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$p
     if(lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$X[p,1]==0) {lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$X_L[p,]=c(0,0)}
     lgtdata_LPS_XL01_NCF[[h]][[choicetask]]$X_PS2 = lgtdata_XL_original_withPS_regionsUpdated_NCF[[h]][[choicetask]]$X_PS2
  }
}

#-------------------------------------------------------------------------------------------------------
# Generate clean file with 24 choice per person
#-------------------------------------------------------------------------------------------------------
 
lgtdata_LPS_XL01_NCF_24ch = list()
j=1
for(h in 1:length(lgtdata_LPS_XL01_NCF))    # for each respondent
{ if(length(lgtdata_LPS_XL01_NCF[[h]])>23)
  {  lgtdata_LPS_XL01_NCF_24ch[[j]] = lgtdata_LPS_XL01_NCF[[h]]
     j=j+1
}}
length(lgtdata_LPS_XL01_NCF_24ch)  # 300 out of 319


#-------------------------------------------------------------------------------------------------------
# Split the file into with and without non-choice sub-files,add intercept column for the withNonChoice sample
#-------------------------------------------------------------------------------------------------------
 
lgtdata_LPS_XL01_NCF_24ch_ForcedChoices = list()
lgtdata_LPS_XL01_NCF_24ch_withNonChoice = list()
nvar_PS2 = ncol(lgtdata_LPS_XL01_NCF_24ch[[1]][[1]]$X_PS2)
for(h in 1:length(lgtdata_LPS_XL01_NCF_24ch))    # for each respondent
{ j=1
  m=1
  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]]=list()
  lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]]=list()
  for(i in 1:length(lgtdata_LPS_XL01_NCF_24ch[[h]]))
  {  
    p = lgtdata_LPS_XL01_NCF_24ch[[h]][[i]]$p
    if(lgtdata_LPS_XL01_NCF_24ch[[h]][[i]]$X_L[p,1]==0)
      {  lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[j]] = lgtdata_LPS_XL01_NCF_24ch[[h]][[i]]
          #  APPEND NON-CHOICE intercept here
         lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[j]]$X_PS2 = cbind(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[j]]$X_PS2,rep(0,p))
         lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[j]]$X_PS2[p,(nvar_PS2+1)]=1  
         j=j+1
      }
    else
      {  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]][[m]] = lgtdata_LPS_XL01_NCF_24ch[[h]][[i]]
         m=m+1
      }
  }
}

# calculate the proportion of no-choice selections
nochoice_count = matrix(0,nrow=length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice),ncol=1)
allchoices_count = 0
for(hh in 1:length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice))
{
  allchoices_count = allchoices_count + length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[hh]])
  for(j in 1:length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[hh]])) { 
    nochoice_count[hh,1] = nochoice_count[hh,1] + ifelse(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[hh]][[j]]$y==lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[hh]][[j]]$p,1,0)
  }
}
table(nochoice_count)
sum(nochoice_count)  # 159 out of 3600  = 4.4%
allchoices_count
table(nochoice_count)[-1]

#-------------------------------------------------------------------------------------------------------
# break down to INSAMPLE and HOLDOUT so that there is only 2 choices in HOLDOUT randomly
#-------------------------------------------------------------------------------------------------------

set.seed(5)
lgtdata_LPS_XL01_NCF_24ch_withNonChoice_INSAMPLE = list()  # used in estimation  (E2-Movie-Singles.NC/FC:NC)
lgtdata_LPS_XL01_NCF_24ch_withNonChoice_HOLDOUT = list()
for(h in 1:length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice))
{  indexHOLDOUT = sort(sample(c(1:length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]])),2,replace=FALSE))
   indexINSAMPLE = setdiff(c(1:length(lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]])),indexHOLDOUT)
   lgtdata_LPS_XL01_NCF_24ch_withNonChoice_INSAMPLE[[h]] = list()
   j=1
   for(r in indexINSAMPLE)    # for each respondent
   {  lgtdata_LPS_XL01_NCF_24ch_withNonChoice_INSAMPLE[[h]][[j]]=lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[r]]
      j=j+1
   }
   lgtdata_LPS_XL01_NCF_24ch_withNonChoice_INSAMPLE[[h]][[1]]$Qualtrics =  lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[1]]$Qualtrics
   j=1
   lgtdata_LPS_XL01_NCF_24ch_withNonChoice_HOLDOUT[[h]] = list()
   for(r in indexHOLDOUT)    # for each respondent
   {  lgtdata_LPS_XL01_NCF_24ch_withNonChoice_HOLDOUT[[h]][[j]] = lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[r]]
      j=j+1
   }
   lgtdata_LPS_XL01_NCF_24ch_withNonChoice_HOLDOUT[[h]][[1]]$Qualtrics =  lgtdata_LPS_XL01_NCF_24ch_withNonChoice[[h]][[1]]$Qualtrics
}
set.seed(5)
lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_INSAMPLE = list()  # used in estimation  (E2-Movie-Singles.NC/FC:FC)
lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_HOLDOUT = list()
for(h in 1:length(lgtdata_LPS_XL01_NCF_24ch_ForcedChoices))  
{  indexHOLDOUT = sort(sample(c(1:length(lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]])),2,replace=FALSE))
   indexINSAMPLE = setdiff(c(1:length(lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]])),indexHOLDOUT)
   lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_INSAMPLE[[h]] = list()
   j=1
   for(r in indexINSAMPLE)    # for each respondent
   {  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_INSAMPLE[[h]][[j]]=lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]][[r]]
      j=j+1
   }
   lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_INSAMPLE[[h]][[1]]$Qualtrics =  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]][[1]]$Qualtrics
   j=1
   lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_HOLDOUT[[h]] = list()
   for(r in indexHOLDOUT)    # for each respondent
   {  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_HOLDOUT[[h]][[j]] = lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]][[r]]
      j=j+1
   }
   lgtdata_LPS_XL01_NCF_24ch_ForcedChoices_HOLDOUT[[h]][[1]]$Qualtrics =  lgtdata_LPS_XL01_NCF_24ch_ForcedChoices[[h]][[1]]$Qualtrics
}

# in case you want to save the workspace
save.image("E2_MovieSingles_finalData.RData")

